QUESTGURU_VERSION = GetAddOnMetadata("QuestGuru", "Version");

QuestGuru_Settings = {};
QuestGuru_Items = {};

QuestGuru_History = {};
QuestGuru_HistoryHeaders = {};
QuestGuru_currHistory = 2;

QGC_Cache = {};
QGC_Active = {};

local QuestGuru_RealmName, QuestGuru_PlayerFaction;

do
	local QuestGuru_Frame = CreateFrame("Frame", "QuestGuru_Frame", UIParent);
	QuestGuru_Frame:SetScript("OnEvent",
		function (self, event, arg1, arg2)
			QuestGuru_OnEvent(self, event, arg1, arg2);
		end);

	QuestGuru_Frame:RegisterEvent("PLAYER_LOGIN");
	QuestGuru_Frame:RegisterEvent("QUEST_COMPLETE");
	QuestGuru_Frame:RegisterEvent("QUEST_LOG_UPDATE");
	QuestGuru_Frame:RegisterEvent("QUEST_QUERY_COMPLETE");
	QuestGuru_Frame:RegisterEvent("QUEST_PROGRESS");
	QuestGuru_Frame:RegisterEvent("UI_INFO_MESSAGE");
	QuestGuru_Frame:RegisterEvent("VARIABLES_LOADED");

	tinsert(UISpecialFrames, "QuestGuru_HistoryFrame");
end

function QuestGuru_OnEvent(self, event, arg1, arg2)
	if (event == "UI_INFO_MESSAGE") then
		QuestGuru_UIInfoMessage(arg1);
	elseif (event == "VARIABLES_LOADED") then
		QuestGuru_HistoryTitleText:SetText("QuestGuru History v"..QUESTGURU_VERSION);
		QuestLogTitleText:SetText(QuestLogTitleText:GetText().." - QuestGuru v"..QUESTGURU_VERSION);
		QuestGuru_LoadVariables();
		wipe(QGC_Cache);
		for i, qh in ipairs(QGC_Both) do tinsert(QGC_Cache, qh); end
		for i, qh in ipairs(getglobal("QGC_"..QuestGuru_PlayerFaction)) do tinsert(QGC_Cache, qh); end
		table.sort(QGC_Cache, function(a, b) return (a.Header < b.Header or ((a.Header == b.Header) and (a.Level < b.Level)) or ((a.Header == b.Header) and (a.Level == b.Level) and (a.Title < b.Title))); end);
		wipe(QGC_Both);
		wipe(QGC_Horde);
		wipe(QGC_Alliance);
	elseif (event == "PLAYER_LOGIN") then
		QueryQuestsCompleted();
	elseif (event == "QUEST_QUERY_COMPLETE") then
		QuestGuru_History = GetQuestsCompleted();
		QuestGuru_History_Update();
	elseif (event=="QUEST_COMPLETE") then
		if (QuestGuru_Settings.AutoComplete and (GetNumQuestChoices() == 0)) then
			QuestRewardCompleteButton_OnClick();
		end
	elseif (event=="QUEST_PROGRESS") then
		QuestGuru_QuestProgressShow();
		if (QuestGuru_Settings.AutoComplete and IsQuestCompletable()) then
			CompleteQuest();
		end
	elseif (event=="QUEST_LOG_UPDATE") then
		wipe(QGC_Active);
		local numEntries, numQuests = GetNumQuestLogEntries();
		for i=1, numEntries do
			local qLink = GetQuestLink(i);
			if (qLink) then
				local qi, qj, questNum = string.find(qLink, (":(%d+):"));
				if (questNum) then QGC_Active[questNum]="true"; end
			end
		end
	end
end

function QuestGuru_HistoryFrame_OnShow(self)
	QueryQuestsCompleted();
end

function QuestGuru_HistoryTitleButton_OnEnter(self, motion)
	getglobal(self:GetName().."Tag"):SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
	getglobal(self:GetName().."Level"):SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);

	if (self.Link ~= nil) then
		GameTooltip:SetOwner(self, "ANCHOR_LEFT");
		GameTooltip:SetHyperlink(self.Link);
		GameTooltip:Show();
	end
end

function QuestGuru_HistoryTitleButton_OnClick(self, button, down)
	local questName = self:GetText();
	local questIndex = self:GetID() + FauxScrollFrame_GetOffset(QuestGuru_HistoryListScrollFrame);
	
	QuestLog_SetHistorySelection(questIndex)
	if (button == "LeftButton") then
		if ( IsShiftKeyDown() ) then
			-- If header then return
			if ( self.isHeader ) then
				return;
			end
			-- Otherwise try to put it into chat
			if ( ChatFrameEditBox:IsVisible() ) then
				if (self.Link) then
--					ChatFrameEditBox:Insert(self.Link);
				else
					ChatFrameEditBox:Insert(strtrim("["..self.Level.."] "..self.Title));
				end
			end
		end
	end	
	QuestGuru_History_Update();
end

function QuestLog_SetHistorySelection(questID)
	QuestGuru_currHistory = questID;

	if ( questID == 0 ) then
		QuestGuru_History_Update();
		return;
	end

	-- Get xml id
	local id = questID - FauxScrollFrame_GetOffset(QuestGuru_HistoryListScrollFrame);

	local titleButton = getglobal("QuestGuru_HistoryTitle"..id);
	local titleButtonTag = getglobal("QuestGuru_HistoryTitle"..id.."Tag");
	local titleButtonStart = getglobal("QuestGuru_HistoryTitle"..id.."Start");

	if ( titleButton.isHeader ) then
		if ( QuestGuru_HistoryHeaders[titleButton.Title] == true ) then
			QuestGuru_HistoryHeaders[titleButton.Title] = false;
		else
			QuestGuru_HistoryHeaders[titleButton.Title] = true;
		end
		QuestGuru_History_Update();
	else
		-- Set newly selected quest and highlight it
		local scrollFrameOffset = FauxScrollFrame_GetOffset(QuestGuru_HistoryListScrollFrame);
		if ( questID > scrollFrameOffset and questID <= (scrollFrameOffset + 27) ) then
			titleButton:LockHighlight();
			titleButtonTag:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
			titleButtonStart:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
			QuestGuru_HistorySkillHighlight:SetVertexColor(titleButton.r, titleButton.g, titleButton.b);
			QuestGuru_HistoryHighlightFrame:SetPoint("TOPLEFT", "QuestGuru_HistoryTitle"..id, "TOPLEFT", 5, 0);
			QuestGuru_HistoryHighlightFrame:Show();
		end
	end
end

function QuestGuru_ExpandCollapse()
	local i, qH;
	local collapsed=true;

	for i, qH in ipairs(QGC_Cache) do
		if (QuestGuru_HistoryHeaders[qH.Header]) then collapsed=false; end
	end
	if (collapsed) then
		for i, qH in ipairs(QGC_Cache) do
			QuestGuru_HistoryHeaders[qH.Header] = true;
		end
--		QuestLog_SetHistorySelection(2);
	else
		for i, qH in ipairs(QGC_Cache) do
			QuestGuru_HistoryHeaders[qH.Header] = false;
		end
--		QuestLog_SetHistorySelection(0);
	end
	FauxScrollFrame_SetOffset(QuestGuru_HistoryListScrollFrame, 0);
	QuestGuru_History_Update();
end

function QuestGuru_History_Update()
	local count=0;
	local headCount = 0;
	local titleCount = 0;
	local currHistory = 1;
	local lastHeader = 0;
	local currHeadCount = 0;
	local currCompleteCount = 0;
	local currHeader = "";
	local questHistory;
	local scrollOffset = FauxScrollFrame_GetOffset(QuestGuru_HistoryListScrollFrame);
	
	local questIndex, questHistoryTitle, questTitleTag, questLevel, questNormalText, questHighlight;
	local questLogTitleText, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, color;
	local numPartyMembers, partyMembersOnQuest, tempWidth, textWidth;
	local strLvl, textLeft;
	
	QuestGuru_HistoryHighlightFrame:Hide();

	for i, questHistory in ipairs(QGC_Cache) do
		if ((strfind(strlower(questHistory.Title),strlower(QuestGuru_HistorySearch:GetText()))) or (strfind(strlower(questHistory.Header),strlower(QuestGuru_HistorySearch:GetText())))) then
			if ((titleCount+headCount >= scrollOffset) or (questHistory.Header ~= currHeader and titleCount+headCount+1==scrollOffset)) then
				if (currHistory <= 27) then
					if ((titleCount+headCount+1) == scrollOffset and questHistory.Header ~= currHeader) then
						currHeader = questHistory.Header;
						headCount = headCount + 1;
					else
						if (questHistory.Header ~= currHeader) then
							if (lastHeader > 0) then
								lastHeadTitle = getglobal("QuestGuru_HistoryTitle"..lastHeader);
								if (QuestGuru_Settings.HeaderQuests) then lastHeadTitle:SetText(currHeader.." ("..currCompleteCount.."/"..currHeadCount..")"); end
							end
							currHeadCount = 0;
							currCompleteCount = 0;
							lastHeader = currHistory;
							questHistoryTitle = getglobal("QuestGuru_HistoryTitle"..currHistory);
							questTitleTag = getglobal("QuestGuru_HistoryTitle"..currHistory.."Tag");
							questLevel = getglobal("QuestGuru_HistoryTitle"..currHistory.."Level");
							questTitleStart = getglobal("QuestGuru_HistoryTitle"..currHistory.."Start");
							questNormalText = getglobal("QuestGuru_HistoryTitle"..currHistory.."NormalText");
							questHighlight = getglobal("QuestGuru_HistoryTitle"..currHistory.."Highlight");
							questHistoryTitle:SetText(questHistory.Header);
							questHistoryTitle.Title=questHistory.Header;
							if (QuestGuru_HistoryHeaders[questHistory.Header] ~= false) then
								questHistoryTitle:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
								QuestGuru_HistoryHeaders[questHistory.Header] = true;
							else
								questHistoryTitle:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
							end
							questHighlight:SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight");
							questHistoryTitle.isHeader = true;
							questHistoryTitle.qID = "";
							questHistoryTitle.Link = nil;
							currHeader = questHistory.Header;
							color = QuestDifficultyColors["header"];
							questTitleTag:SetText("");
							questTitleTag:SetTextColor(color.r, color.g, color.b);
							questTitleStart:SetText("");
							questTitleStart:SetTextColor(color.r, color.g, color.b);
							questLevel:SetText("");
							questHistoryTitle:SetNormalFontObject(color.font);
							questHistoryTitle.r = color.r;
							questHistoryTitle.g = color.g;
							questHistoryTitle.b = color.b;
							questHistoryTitle:Show();
							textLeft = 20;
							questNormalText:SetPoint("LEFT", textLeft, 0);
							headCount = headCount + 1;
							currHistory = currHistory + 1;
						end
					end

					if ((currHistory <= 27) and (QuestGuru_HistoryHeaders[questHistory.Header] ~= false)) then
						questHistoryTitle = getglobal("QuestGuru_HistoryTitle"..currHistory);
						questTitleTag = getglobal("QuestGuru_HistoryTitle"..currHistory.."Tag");
						questLevel = getglobal("QuestGuru_HistoryTitle"..currHistory.."Level");
						questTitleStart = getglobal("QuestGuru_HistoryTitle"..currHistory.."Start");
						questNormalText = getglobal("QuestGuru_HistoryTitle"..currHistory.."NormalText");
						questHighlight = getglobal("QuestGuru_HistoryTitle"..currHistory.."Highlight");
		
						strLvl = questHistory.Level;
						if (strLvl == -1) then strLvl="*"; end
						if (questHistory.Daily) then
							strLvl = strLvl.."Y";
						end
						if (questHistory.Tag ~= nil) then
							if ((questHistory.Tag == "Group") and (questHistory.SuggestedGroup > 0)) then
								strLvl = strLvl.."G"..questHistory.SuggestedGroup;
							elseif (questHistory.Tag == "Dungeon") then
								strLvl = strLvl.."D";
							else
								strLvl = strLvl.."+"
							end
						end
						questLevel:SetText(strLvl);
						local s = questHistory.Title;
						if (questHistory.Count ~= nil and questHistory.Daily) then
							s = s.." ("..questHistory.Count..")";
						end
						questHistoryTitle:SetText(s);
						questHistoryTitle.Title = questHistory.Title;
						QuestGuru_QuestLogDummyText:SetText(s);

						if (QuestGuru_Settings.ShowLevels == nil) then
						    QuestGuru_Settings.ShowLevels = {};
						end
						if (QuestGuru_Settings.ShowLevels.History) then
						    questLevel:Show();
						    textLeft = 40;
						else
						    questLevel:Hide();
						    textLeft = 16;
						end
						questNormalText:SetPoint("LEFT", textLeft, 0);
						questNormalText:SetWidth(288 + (40 - textLeft));

						questHistoryTitle:SetNormalTexture("");
						questHighlight:SetTexture("");
		
						-- Save if its a header or not
						questHistoryTitle.isHeader = false;
						
						questHistoryTitle.Level = questHistory.Level;
						
						questHistoryTitle.Link = "\124cffffff00\124Hquest:"..questHistory.QuestID..":"..questHistory.Level.."\124h["..questHistory.Title.."]\124h\124r";
		
						if (QuestGuru_History[questHistory.QuestID+0]) then
							questTitleTag:SetText("("..COMPLETE..")");
							--currCompleteCount = currCompleteCount + 1;
						elseif (QGC_Active[questHistory.QuestID]) then
							questTitleTag:SetText("("..QG_ACTIVE..")");
						else
							questTitleTag:SetText("");
						end
						
						questTitleStart:SetText(questHistory.Start);
		
						-- Color the quest title and highlight according to the difficulty level
						local playerLevel = UnitLevel("player");
						color = GetQuestDifficultyColor(questHistory.Level);
						questTitleTag:SetTextColor(color.r, color.g, color.b);
						questTitleStart:SetTextColor(color.r, color.g, color.b);
						questHistoryTitle:SetNormalFontObject(color.font);
						questLevel:SetTextColor(color.r, color.g, color.b);
						questHistoryTitle.r = color.r;
						questHistoryTitle.g = color.g;
						questHistoryTitle.b = color.b;
						questHistoryTitle:Show();
			
						-- Place the highlight and lock the highlight state
						if ( QuestGuru_currHistory == currHistory + scrollOffset ) then
							QuestGuru_HistoryHighlightFrame:SetPoint("TOPLEFT", "QuestGuru_HistoryTitle"..currHistory, "TOPLEFT", 0, 0);
							QuestGuru_HistorySkillHighlight:SetVertexColor(questHistoryTitle.r, questHistoryTitle.g, questHistoryTitle.b);
							QuestGuru_HistoryHighlightFrame:Show();
							questTitleTag:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
							questTitleStart:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
							questLevel:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
							questHistoryTitle:LockHighlight();
						else
							questHistoryTitle:UnlockHighlight();
						end

						currHistory = currHistory + 1;
					end
				else
					if (questHistory.Header ~= currHeader) then
						if (lastHeader > 0) then
							lastHeadTitle = getglobal("QuestGuru_HistoryTitle"..lastHeader);
							if (QuestGuru_Settings.HeaderQuests) then lastHeadTitle:SetText(currHeader.." ("..currCompleteCount.."/"..currHeadCount..")"); end
						end
						currHeadCount = 0;
						lastHeader = 0;
						currHeader = questHistory.Header;
						headCount = headCount + 1;
					end
				end
			else
				if (questHistory.Header ~= currHeader) then
					if (lastHeader > 0) then
						lastHeadTitle = getglobal("QuestGuru_HistoryTitle"..lastHeader);
						if (QuestGuru_Settings.HeaderQuests) then lastHeadTitle:SetText(currHeader.." ("..currCompleteCount.."/"..currHeadCount..")"); end
					end
					currHeadCount = 0;
					lastHeader = 0;
					currHeader = questHistory.Header;
					headCount = headCount + 1;
				end
			end
			if (QuestGuru_HistoryHeaders[questHistory.Header] ~= false) then
				titleCount = titleCount + 1;
			end
			currHeadCount = currHeadCount + 1;
			if (QuestGuru_History[questHistory.QuestID+0]) then currCompleteCount = currCompleteCount + 1; end
			count = count + 1;
		end
	end
	if (lastHeader > 0) then
		lastHeadTitle = getglobal("QuestGuru_HistoryTitle"..lastHeader);
		if (QuestGuru_Settings.HeaderQuests) then lastHeadTitle:SetText(currHeader.." ("..currCompleteCount.."/"..currHeadCount..")"); end
	end

	-- ScrollFrame update
	FauxScrollFrame_Update(QuestGuru_HistoryListScrollFrame, titleCount + headCount, 27, 16, nil, nil, nil, QuestGuru_HistoryHighlightFrame, 293, 316 )

	for i=currHistory, 27 do
		getglobal("QuestGuru_HistoryTitle"..i):Hide();
	end
end

function QuestGuru_TitleButtonCheckBox_OnClick(self, button, down)
	local index = self:GetParent():GetID();
	local questLogTitleText, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily = GetQuestLogTitle(index);

	if (self:GetChecked()) then
		if (isHeader) then
			QuestGuru_AddQuestWatchHeader(questLogTitleText);
		else
			if (not AddQuestWatch(index)) then
				self:SetChecked(0);
			end
		end
	else
		if (isHeader) then
			QuestGuru_RemoveQuestWatchHeader(questLogTitleText);
		else
			RemoveQuestWatch(index);
		end
	end
	WatchFrame_Update();
	QuestLog_Update();
end

function QuestGuru_AddQuestWatchHeader(questHeader)
	local numEntries, numQuests = GetNumQuestLogEntries();
	local inHeader = false;

	for questIndex=1, numEntries do
		local questLogTitleText, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily = GetQuestLogTitle(questIndex);
		if (isHeader) then
			if (questLogTitleText == questHeader) then
				inHeader = true;
			else
				inHeader = false;
			end
		end
		if (inHeader and (not isHeader)) then
			AddQuestWatch(questIndex);
		end
	end
end

function IsHeaderWatched(questHeader)
	local numEntries, numQuests = GetNumQuestLogEntries();
	local inHeader = false;
	local headerWatched = true;

	for questIndex=1, numEntries do
		local questLogTitleText, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily = GetQuestLogTitle(questIndex);
		if (isHeader) then
			if (questLogTitleText == questHeader) then
				inHeader = true;
			else
				inHeader = false;
			end
		end
		if (inHeader and (not isHeader)) then
			headerWatched = (headerWatched and IsQuestWatched(questIndex));
		end
	end
	return headerWatched;
end

function QuestGuru_RemoveQuestWatchHeader(questHeader)
	local numEntries, numQuests = GetNumQuestLogEntries();
	local inHeader = false;

	for questIndex=1, numEntries do
		local questLogTitleText, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily = GetQuestLogTitle(questIndex);
		if (isHeader) then
			if (questLogTitleText == questHeader) then
				inHeader = true;
			else
				inHeader = false;
			end
		end
		if (inHeader and (not isHeader)) then
			RemoveQuestWatch(questIndex);
		end
	end
end

local prevParent;
local function QuestGuru_QuestLog_HighlightQuest(questLogTitle)
	if ( prevParent and prevParent ~= questLogTitle ) then
		-- set prev quest's colors back to normal
		prevParent.level:SetTextColor(prevParent.r, prevParent.g, prevParent.b);
	end
	if ( questLogTitle ) then
		-- highlight the quest's colors
		questLogTitle.level:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
	end
	prevParent = questLogTitle;
end

function QuestGuru_SetSelection(questIndex)
	if ( questIndex == 0 ) then
		return;
	end

	-- find and highlight the selected quest
	local buttons = QuestLogScrollFrame.buttons;
	local numButtons = #buttons;
	local min = 1;
	local max = #buttons;
	local mid;
	local titleButton, id;
	while ( min <= max ) do
		mid = bit.rshift(min + max, 1);
		titleButton = buttons[mid];
		id = titleButton:GetID();
		if ( id == questIndex ) then
			QuestGuru_QuestLog_HighlightQuest(titleButton);
			break;
		end
		if ( id > questIndex ) then
			max = mid - 1;
		else
			min = mid + 1;
		end
	end
end
hooksecurefunc("QuestLog_SetSelection", QuestGuru_SetSelection);

local QG_QuestLogUpdate = QuestLog_Update;
function QuestLog_Update()
	QG_QuestLogUpdate();
	local buttons = QuestLogScrollFrame.buttons;
	local numButtons = #buttons;
	local scrollOffset = HybridScrollFrame_GetOffset(QuestLogScrollFrame);
	local questLogSelection = GetQuestLogSelection();

	local questIndex, questLogTitle, questTitleTag, questNumGroupMates, questNormalText, questCheck;
	local title, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily;
	for i=1, numButtons do
		questLogTitle = buttons[i];
		questIndex = questLogTitle:GetID();
		title, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily = GetQuestLogTitle(questIndex);
		questNumGroupMates = questLogTitle.groupMates;
		questNumGroupMates:SetPoint("LEFT", 6, 0);
		if (questLogTitle.level == nil) then
			questLogTitle.level = questLogTitle:CreateFontString(questLogTitle:GetName().."Level", "BACKGROUND", "GameFontNormalSmall");
			questLogTitle.tag:SetFontObject("GameFontNormalSmall");
			questLogTitle.level:SetPoint("LEFT", 28, 0);
			questLogTitle.level:SetWidth(32);
			questLogTitle.level:SetJustifyH("CENTER");
			questLogTitle.level.parentKey = "level";
			questLogTitle:HookScript("OnEnter", function(self)
					self.level:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
				end);
			questLogTitle:HookScript("OnLeave", function(self)
					if ( self:GetID() ~= QuestLogFrame.selectedIndex ) then
						self.level:SetTextColor(self.r, self.g, self.b);
					end
				end);
		end
		if (questLogTitle.checkBox == nil) then
			questLogTitle.checkBox = CreateFrame("CheckButton", questLogTitle:GetName().."CheckBox", questLogTitle, "OptionsCheckButtonTemplate");
			questLogTitle.checkBox:SetHeight(16);
			questLogTitle.checkBox:SetWidth(16);
			questLogTitle.checkBox:SetPoint("LEFT", 16, 0);
			questLogTitle.checkBox:SetHitRectInsets(0, 0, 0, 0);
			questLogTitle.checkBox:SetScript("OnClick", QuestGuru_TitleButtonCheckBox_OnClick);
		end
		questLogTitle.check:Hide();

		questLevel = questLogTitle.level;
		if (isHeader) then
			questLevel:SetText("");
			questLogTitle.normalText:SetPoint("LEFT", 32, 0);
			if (IsHeaderWatched(title)) then
				questLogTitle.checkBox:SetChecked(1);
			else
				questLogTitle.checkBox:SetChecked(0);
			end
		else
			local strLvl = level
			if (isDaily) then
				strLvl = strLvl.."Y";
			end
			if (questTag ~= nil) then
				if ((questTag == "Group") and (suggestedGroup > 0)) then
					strLvl = strLvl.."G"..suggestedGroup;
				elseif (questTag == "Dungeon") then
					strLvl = strLvl.."D";
				else
					strLvl = strLvl.."+"
				end
			end

			if (QuestGuru_Settings.ShowLevels == nil) then
				QuestGuru_Settings.ShowLevels = {};
			end
			if (QuestGuru_Settings.ShowLevels.QuestLog) then
				questLevel:Show();
				textLeft = 52;
			else
				questLevel:Hide();
				textLeft = 32;
			end
			questLevel:SetText(strLvl);
			questLogTitle.normalText:SetPoint("LEFT", textLeft, 0);
			local color = GetQuestDifficultyColor(level);
			questLevel:SetTextColor(color.r, color.g, color.b);
			if ( questLogSelection == questIndex ) then
				QuestGuru_QuestLog_HighlightQuest(questLogTitle);
			end
			if (IsQuestWatched(questIndex)) then
				questLogTitle.checkBox:SetChecked(1);
			else
				questLogTitle.checkBox:SetChecked(0);
			end
		end
		QuestLogTitleButton_Resize(questLogTitle);
	end
end
QuestLogScrollFrame.update = QuestLog_Update;

function QuestGuru_ShowObjectives(template, parentFrame, acceptButton, cancelButton, material)
	local i, lastObjective;
	local numObjectives = GetNumQuestLeaderBoards();
	
	for i=1, MAX_OBJECTIVES do
		getglobal("QuestGuru_QuestLogObjectiveItem"..i):Hide();
		getglobal("QuestGuru_QuestLogObjectiveItem"..i).timerEnabled = false;
	end

	if (QuestGuru_Settings.ShowObjItemIcons) then
		local currObjItem = 0;
		for i=1, numObjectives, 1 do
			local text, type, finished = GetQuestLogLeaderBoard(i);
			local count;
			if (type == "item") then
				local questObj = getglobal("QuestInfoObjective"..i);
				if ( not text or strlen(text) == 0 ) then
					text = type;
				end
				if ( finished ) then
					questObj:SetTextColor(0.2, 0.2, 0.2);
					text = text.." ("..COMPLETE..")";
				else
					questObj:SetTextColor(0, 0, 0);
				end
				questObj:SetText(text);
				questObj:Show();
				questObj:SetAlpha(0);
				lastObjective = questObj;

				currObjItem = currObjItem + 1;
				local questItem = getglobal("QuestGuru_QuestLogObjectiveItem"..currObjItem);
				questItem.type = "required";
				local x, y, name, numItems, numNeeded = string.find(text, QG_ITEM_REQ_STR);
				local iiName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, texture = GetItemInfo(name);

				if ((not itemLink) and QuestGuru_Items[QuestGuru_RealmName][name]) then
					iiName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, texture = GetItemInfo(QuestGuru_Items[QuestGuru_RealmName][name]);
				end
				if (not itemLink) then
					texture = "Interface\\Icons\\INV_Misc_QuestionMark";
					count = 0;
					SetItemButtonTextureVertexColor(questItem, 0.4, 0.4, 0.4);
					QuestGuru_CommSendPGMessage("0301", name);
				else
					if (texture == nil) then
						GameTooltip:SetHyperlink(itemLink);
						GameTooltip:Show();
						GameTooltip:Hide();
						questItem.timerEnabled = true;
					end
					count = GetItemCount(itemLink);
					SetItemButtonTextureVertexColor(questItem, 0.8, 0.8, 0.8);
				end
				questItem:SetID(currObjItem)
				-- For the tooltip
				questItem.rewardType = "item"
				questItem.link = itemLink;
				getglobal("QuestGuru_QuestLogObjectiveItem"..currObjItem.."Name"):SetText(name);
				getglobal("QuestGuru_QuestLogObjectiveItem"..currObjItem.."Count"):SetText(count.." /"..numNeeded);
				SetItemButtonTexture(questItem, texture);
				SetItemButtonNameFrameVertexColor(questItem, 1, 1, 1);
				if ( currObjItem  > 1 ) then
					if ( mod(currObjItem ,2) == 1 ) then
						questItem:SetPoint("TOPLEFT", "QuestGuru_QuestLogObjectiveItem"..(currObjItem - 2), "BOTTOMLEFT", 0, -2);
						lastObjective = questItem;
					else
						questItem:SetPoint("TOPLEFT", "QuestGuru_QuestLogObjectiveItem"..(currObjItem - 1), "TOPRIGHT", 1, 0);
						lastObjective = getglobal("QuestGuru_QuestLogObjectiveItem"..(currObjItem)-1);
					end
				else
					if (i > 1) then
						questItem:SetPoint("TOPLEFT", "QuestInfoObjective"..(i - 1), "BOTTOMLEFT", 0, -5);
					else
						questItem:SetPoint("TOPLEFT");
					end
					lastObjective = questItem;
				end
				questItem:Show();
			end
		end
		if ( lastObjective ) then
			QuestInfoObjectivesFrame:SetHeight(QuestInfoObjectivesFrame:GetTop() - lastObjective:GetTop() + lastObjective:GetHeight());
		end
	end
end
hooksecurefunc("QuestInfo_Display", QuestGuru_ShowObjectives);

function QuestGuru_UIInfoMessage(arg1)
	local questID=0;
	local sound, announceText;
	local numEntries, numQuests = GetNumQuestLogEntries();
	local i, ii, jj, itemName, numDone, numNeeded;
	local questLogTitleText, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily;

	if (arg1 == nil) then return; end

	for questID=1, numEntries, 1 do
		questLogTitleText, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily = GetQuestLogTitle(questID);
		local numObjectives = GetNumQuestLeaderBoards(questID);
		local currQuest = false;
		local questComplete = true;
		sound = nil;
		announceText = nil;

		if (numObjectives ~= nil) then
			for i=1, numObjectives, 1 do
				local text, type, finished = GetQuestLogLeaderBoard(i,questID);
				local qi, qj, qitemName, qnumDone, qnumNeeded = string.find(text, QG_ITEM_REQ_STR);
				if ((type == "item") or (type == "monster") or (type == "object")) then
					if (type == "object") then type = "item"; end
					ii, jj, itemName, numDone, numNeeded = string.find(arg1, QG_ITEM_REQ_STR);
					if ((itemName == qitemName) and (numNeeded == qnumNeeded)) then
						if (QuestGuru_Settings.Sounds.Progress.Enabled) then sound = QuestGuru_Settings.Sounds.Progress.Sound; end
						if (QuestGuru_Settings.Announce.item.Enabled and (type == "item")) then announceText = QuestGuru_Settings.Announce.item.Text; end
						if (QuestGuru_Settings.Announce.monster.Enabled and (type == "monster")) then announceText = QuestGuru_Settings.Announce.monster.Text; end
						if (numDone == numNeeded) then
							if (QuestGuru_Settings.Sounds.ObjComplete.Enabled) then sound = QuestGuru_Settings.Sounds.ObjComplete.Sound; end
							if (announceText) then announceText = announceText.." ($complete)"; end
							finished = true;
						end
						currQuest = true;
					end
				elseif (type == "event") then
					itemName = text;
					numDone=1;
					numNeeded=1;
					if (arg1 == text.." ("..COMPLETE..")") then
						if (QuestGuru_Settings.Sounds.ObjComplete.Enabled) then sound = QuestGuru_Settings.Sounds.ObjComplete.Sound; end
						if (QuestGuru_Settings.Announce.event.Enabled) then announceText = QuestGuru_Settings.Announce.event.Text; end
						currQuest = true;
						finished = true;
					end
				end
				if (not finished) then questComplete = false; end
			end
		end
		if (currQuest) then
			if (questComplete) then
				if (QuestGuru_Settings.Sounds.QuestComplete.Enabled) then sound = QuestGuru_Settings.Sounds.QuestComplete.Sound; end
				if (QuestGuru_Settings.Announce.Quest.Enabled) then announceText = QuestGuru_Settings.Announce.Quest.Text; end
				UIErrorsFrame:AddMessage(QG_QUEST_COMPLETE, 1, 1, 0, 1, 5);
			end
			QuestGuru_PlayExtraSound(sound);
			QuestGuru_DoAnnounce(announceText, numDone, numNeeded, itemName, questLogTitleText, arg1, questID);
			QuestGuru_CommAddTimer(1, 1, QuestGuru_SendQuestStatus, questID, false);
		end
	end
end

function QuestGuru_PlayExtraSound(sound)
	if (sound == nil) then return; end
	
	if (QuestGuru_Settings.Sounds.Enabled) then
		if ((string.sub(sound, -4) == ".wav") or (string.sub(sound, -4) == ".mp3")) then
			PlaySoundFile(sound);
		else
			PlaySound(sound);
		end
	end
end

function QuestGuru_QuestProgressShow()
	local questTitle = GetTitleText();
	
	QuestGuru_UpdateProgressFrame(questTitle, GetGossipActiveQuests());

	local i;
	for i=1, GetNumQuestItems() do
		local name, texture, numItems, quality, isUsable = GetQuestItemInfo("required", i);
		local link = GetQuestItemLink("required", i);
		if (name and link) then
			QuestGuru_Items[QuestGuru_RealmName][name] = link;
			_G["QuestProgressItem"..i].link = link;
			_G["QuestProgressItem"..i]:SetScript("OnEnter", function (self)
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
				GameTooltip:SetHyperlink(self.link);
			end);
		end
	end
end

function QuestGuru_UpdateProgressFrame(questTitle, ...)
	local i;

	for i=1, select("#", ...), 3 do
	    if (select(i, ...) == questTitle) then
	        QuestProgressTitleText:SetText("["..select(i+1, ...).."] "..questTitle);
		end
	end
end

function QuestGuru_LoadVariables()
		QuestGuru_Player = UnitName("player");
		QuestGuru_PlayerFaction = UnitFactionGroup("player");
		QuestGuru_RealmName = GetRealmName();

		if (QuestGuru_Items == nil) then
			QuestGuru_Items = {};
		end
		if (QuestGuru_Items[QuestGuru_RealmName] == nil) then
			QuestGuru_Items[QuestGuru_RealmName] = {};
		end
		
		if (QuestGuru_Settings == nil) then
			QuestGuru_Settings = {};
		end
		QuestGuru_QuestAnnounceLoadSettings();
		QuestGuru_OptionsSoundsLoadSettings();
		
		if (QuestGuru_Settings.AutoComplete ~= true) then
			QuestGuru_Settings.AutoComplete = false;
		end
		QuestGuru_OptionsFrameAutoCompleteToggle:SetChecked(QuestGuru_Settings.AutoComplete);
		
		if (QuestGuru_Settings.ShowObjItemIcons ~= true) then
			QuestGuru_Settings.ShowObjItemIcons = false;
		end
		QuestGuru_OptionsFrameShowObjItemIconsToggle:SetChecked(QuestGuru_Settings.ShowObjItemIcons);
		
		if (QuestGuru_Settings.DisableComm ~= true) then
		    QuestGuru_Settings.DisableComm = false;
		end
		QuestGuru_OptionsFrameDisableCommToggle:SetChecked(QuestGuru_Settings.DisableComm);

		if (QuestGuru_Settings.HeaderQuests ~= false) then
		    QuestGuru_Settings.HeaderQuests = true;
		end
		QuestGuru_OptionsFrameShowHeaderQuestsToggle:SetChecked(QuestGuru_Settings.HeaderQuests);

		if (QuestGuru_Settings.ShowLevels == nil) then
			QuestGuru_Settings.ShowLevels = {};
		end
		if (QuestGuru_Settings.ShowLevels.QuestLog ~= false) then
			QuestGuru_Settings.ShowLevels.QuestLog = true;
		end
		QuestGuru_OptionsFrameShowLevelsQuestLogToggle:SetChecked(QuestGuru_Settings.ShowLevels.QuestLog);
		if (QuestGuru_Settings.ShowLevels.History ~= false) then
			QuestGuru_Settings.ShowLevels.History = true;
		end
		QuestGuru_OptionsFrameShowLevelsHistoryToggle:SetChecked(QuestGuru_Settings.ShowLevels.History);
end

function QuestGuru_Echo(msg, always)
	local info = ChatTypeInfo["SYSTEM"];
	if (always or ((QuestGuru_Settings) and (QuestGuru_Settings.DebugLevel) and (QuestGuru_Settings.DebugLevel > 0))) then
		DEFAULT_CHAT_FRAME:AddMessage(msg, info.r, info.g, info.b, info.id);
	end
end
